@@ -15,6 +15,7 @@ from glob import glob
 from setuptools import Extension as _Extension
 from setuptools.command.build_ext import build_ext
 
+THIS_DIR = os.path.dirname(__file__)
 
 ## Exported configurations
 
@@ -24,7 +25,7 @@ WIN = sys.platform.startswith('win')
 RUNNING_ON_TRAVIS = os.environ.get('TRAVIS')
 RUNNING_ON_APPVEYOR = os.environ.get('APPVEYOR')
 RUNNING_ON_CI = RUNNING_ON_TRAVIS or RUNNING_ON_APPVEYOR
-
+RUNNING_FROM_CHECKOUT = os.path.isdir(os.path.join(THIS_DIR, ".git"))
 
 
 LIBRARIES = []
@@ -37,8 +38,6 @@ if WIN:
 
 ### File handling
 
-THIS_DIR = os.path.dirname(__file__)
-
 def quoted_abspath(*segments):
     return '"' + os.path.abspath(os.path.join(*segments)) + '"'
 
@@ -78,7 +77,7 @@ def _parse_environ(key):
     value = value.lower().strip()
     if value in ('1', 'true', 'on', 'yes'):
         return True
-    elif value in ('0', 'false', 'off', 'no'):
+    if value in ('0', 'false', 'off', 'no'):
         return False
     raise ValueError('Environment variable %r has invalid value %r. '
                      'Please set it to 1, 0 or an empty string' % (key, value))
@@ -154,26 +153,49 @@ def system(cmd, cwd=None, env=None, **kwargs):
 
 # Cython
 
+# Based on code from
+# http://cython.readthedocs.io/en/latest/src/reference/compilation.html#distributing-cython-modules
+def _dummy_cythonize(extensions, **_kwargs):
+    for extension in extensions:
+        sources = []
+        for sfile in extension.sources:
+            path, ext = os.path.splitext(sfile)
+            if ext in ('.pyx', '.py'):
+                ext = '.c'
+                sfile = path + ext
+            sources.append(sfile)
+        extension.sources[:] = sources
+    return extensions
 
 try:
     from Cython.Build import cythonize
 except ImportError:
-    # The .c files had better already exist. Based on code from
-    # http://cython.readthedocs.io/en/latest/src/reference/compilation.html#distributing-cython-modules
-    def cythonize(extensions, **_kwargs):
-        for extension in extensions:
-            sources = []
-            for sfile in extension.sources:
-                path, ext = os.path.splitext(sfile)
-                if ext in ('.pyx', '.py'):
-                    ext = '.c'
-                    sfile = path + ext
-                sources.append(sfile)
-            extension.sources[:] = sources
-        return extensions
+    # The .c files had better already exist.
+    cythonize = _dummy_cythonize
 
 def cythonize1(ext):
-    new_ext = cythonize([ext], include_path=['src/gevent', 'src/gevent/libev', 'src/gevent/resolver'])[0]
+    try:
+        new_ext = cythonize(
+            [ext],
+            include_path=['src/gevent', 'src/gevent/libev', 'src/gevent/resolver'],
+            annotate=True,
+            compiler_directives={
+                'language_level': '3str',
+                'always_allow_keywords': False,
+                'infer_types': True,
+                'nonecheck': False,
+            }
+        )[0]
+    except ValueError:
+        # 'invalid literal for int() with base 10: '3str'
+        # This is seen when an older version of Cython is installed.
+        # It's a bit of a chicken-and-egg, though, because installing
+        # from dev-requirements first scans this egg for its requirements
+        # before doing any updates.
+        import traceback
+        traceback.print_exc()
+        new_ext = _dummy_cythonize([ext])[0]
+
     for optional_attr in ('configure', 'optional'):
         if hasattr(ext, optional_attr):
             setattr(new_ext, optional_attr,
