@@ -667,7 +667,7 @@ def test_main(verbose=None):
     # XXX: gevent: On windows with pypy2, some of these
     # tests are incredibly slow or hang in shutdown for unknown
     # reasons
-    import greentest
+    import gevent.testing as greentest
     MySimpleHTTPRequestHandlerTestCase = SimpleHTTPRequestHandlerTestCase
     MySimpleHTTPServerTestCase = SimpleHTTPServerTestCase
     MyCGIHTTPServerTestCase = CGIHTTPServerTestCase
