@@ -9,15 +9,15 @@ from test.support.script_helper import assert_python_ok, assert_python_failure
 
 import random
 import sys
-import _thread
-import threading
+_thread = import_module('_thread')
+threading = import_module('threading')
 import time
 import unittest
 import weakref
 import os
 import subprocess
 
-import lock_tests # gevent: use our local copy
+from test import lock_tests
 from test import support
 
 
@@ -25,7 +25,8 @@ from test import support
 # #12316 and #11870), and fork() from a worker thread is known to trigger
 # problems with some operating systems (issue #3863): skip problematic tests
 # on platforms known to behave badly.
-platforms_to_skip = ('netbsd5', 'hp-ux11')
+platforms_to_skip = ('freebsd4', 'freebsd5', 'freebsd6', 'netbsd5',
+                     'hp-ux11')
 
 
 # A trivial mutable counter.
@@ -124,17 +125,16 @@ class ThreadTests(BaseTestCase):
             done.set()
         done = threading.Event()
         ident = []
-        with support.wait_threads_exit():
-            tid = _thread.start_new_thread(f, ())
-            done.wait()
-            self.assertEqual(ident[0], tid)
+        _thread.start_new_thread(f, ())
+        done.wait()
+        self.assertIsNotNone(ident[0])
         # Kill the "immortal" _DummyThread
         del threading._active[ident[0]]
 
-    # run with a small(ish) thread stack size (256 KiB)
+    # run with a small(ish) thread stack size (256kB)
     def test_various_ops_small_stack(self):
         if verbose:
-            print('with 256 KiB thread stack size...')
+            print('with 256kB thread stack size...')
         try:
             threading.stack_size(262144)
         except _thread.error:
@@ -143,10 +143,10 @@ class ThreadTests(BaseTestCase):
         self.test_various_ops()
         threading.stack_size(0)
 
-    # run with a large thread stack size (1 MiB)
+    # run with a large thread stack size (1MB)
     def test_various_ops_large_stack(self):
         if verbose:
-            print('with 1 MiB thread stack size...')
+            print('with 1MB thread stack size...')
         try:
             threading.stack_size(0x100000)
         except _thread.error:
@@ -165,10 +165,9 @@ class ThreadTests(BaseTestCase):
 
         mutex = threading.Lock()
         mutex.acquire()
-        with support.wait_threads_exit():
-            tid = _thread.start_new_thread(f, (mutex,))
-            # Wait for the thread to finish.
-            mutex.acquire()
+        tid = _thread.start_new_thread(f, (mutex,))
+        # Wait for the thread to finish.
+        mutex.acquire()
         self.assertIn(tid, threading._active)
         self.assertIsInstance(threading._active[tid], threading._DummyThread)
         #Issue 29376
@@ -182,7 +181,6 @@ class ThreadTests(BaseTestCase):
         ctypes = import_module("ctypes")
 
         set_async_exc = ctypes.pythonapi.PyThreadState_SetAsyncExc
-        set_async_exc.argtypes = (ctypes.c_ulong, ctypes.py_object)
 
         class AsyncExc(Exception):
             pass
@@ -191,11 +189,9 @@ class ThreadTests(BaseTestCase):
 
         # First check it works when setting the exception from the same thread.
         tid = threading.get_ident()
-        self.assertIsInstance(tid, int)
-        self.assertGreater(tid, 0)
 
         try:
-            result = set_async_exc(tid, exception)
+            result = set_async_exc(ctypes.c_long(tid), exception)
             # The exception is async, so we might have to keep the VM busy until
             # it notices.
             while True:
@@ -241,7 +237,7 @@ class ThreadTests(BaseTestCase):
         # Try a thread id that doesn't make sense.
         if verbose:
             print("    trying nonsensical thread id")
-        result = set_async_exc(-1, exception)
+        result = set_async_exc(ctypes.c_long(-1), exception)
         self.assertEqual(result, 0)  # no thread states modified
 
         # Now raise an exception in the worker thread.
@@ -254,7 +250,7 @@ class ThreadTests(BaseTestCase):
         self.assertFalse(t.finished)
         if verbose:
             print("    attempting to raise asynch exception in worker")
-        result = set_async_exc(t.id, exception)
+        result = set_async_exc(ctypes.c_long(t.id), exception)
         self.assertEqual(result, 1) # one thread state modified
         if verbose:
             print("    waiting for worker to say it caught the exception")
@@ -426,7 +422,7 @@ class ThreadTests(BaseTestCase):
         t.daemon = True
         self.assertIn('daemon', repr(t))
 
-    def test_daemon_param(self):
+    def test_deamon_param(self):
         t = threading.Thread()
         self.assertFalse(t.daemon)
         t = threading.Thread(daemon=False)
@@ -546,35 +542,6 @@ class ThreadTests(BaseTestCase):
         self.assertEqual(err, b"")
         self.assertEqual(data, "Thread-1\nTrue\nTrue\n")
 
-    def test_main_thread_during_shutdown(self):
-        # bpo-31516: current_thread() should still point to the main thread
-        # at shutdown
-        code = """if 1:
-            import gc, threading
-
-            main_thread = threading.current_thread()
-            assert main_thread is threading.main_thread()  # sanity check
-
-            class RefCycle:
-                def __init__(self):
-                    self.cycle = self
-
-                def __del__(self):
-                    print("GC:",
-                          threading.current_thread() is main_thread,
-                          threading.main_thread() is main_thread,
-                          threading.enumerate() == [main_thread])
-
-            RefCycle()
-            gc.collect()  # sanity check
-            x = RefCycle()
-        """
-        _, out, err = assert_python_ok("-c", code)
-        data = out.decode()
-        self.assertEqual(err, b"")
-        self.assertEqual(data.splitlines(),
-                         ["GC: True True True"] * 2)
-
     def test_tstate_lock(self):
         # Test an implementation detail of Thread objects.
         started = _thread.allocate_lock()
@@ -1137,6 +1104,14 @@ class TimerTests(BaseTestCase):
 class LockTests(lock_tests.LockTests):
     locktype = staticmethod(threading.Lock)
 
+    @unittest.skip("not on gevent")
+    def test_locked_repr(self):
+        pass
+
+    @unittest.skip("not on gevent")
+    def test_repr(self):
+        pass
+
 class PyRLockTests(lock_tests.RLockTests):
     locktype = staticmethod(threading._PyRLock)
 
@@ -1147,6 +1122,11 @@ class CRLockTests(lock_tests.RLockTests):
 class EventTests(lock_tests.EventTests):
     eventtype = staticmethod(threading.Event)
 
+    @unittest.skip("not on gevent")
+    def test_reset_internal_locks(self):
+        # XXX: gevent: this appears to have gone away by 3.6.3
+        pass
+
 class ConditionAsRLockTests(lock_tests.RLockTests):
     # Condition uses an RLock by default and exports its API.
     locktype = staticmethod(threading.Condition)
